import argparse
from pathlib import Path
from .changelog_helpers.helpers import (
    add_changelog_path_argument,
    add_include_unreleased_argument,
    sanitize_changelog_path,
    read_changelog_path,
    get_latest_version,
)
from .changelog_helpers.summarize import get_summary


def add_get_parser(sub_commands_parser: argparse.ArgumentParser):
    """
    Define command line arguments for get feature.
    """

    get_parser = sub_commands_parser.add_parser("get", help="Check and display the latest version of a changelog file.")
    # Add argument to include Unreleased section of changelog or not
    add_include_unreleased_argument(get_parser)
    # Add argument to be able to choose another path for changelog file
    add_changelog_path_argument(get_parser)

    return get_parser


def get(arguments: argparse.Namespace):
    """
    Return latest version of a changelog file.
    """

    path = sanitize_changelog_path(Path(arguments.changelog_path))

    changelog_file = read_changelog_path(path)

    print(get_latest_version(changelog_file, arguments.include_unreleased))


def add_summarize_parser(sub_commands_parser: argparse.ArgumentParser):
    """
    Define command line arguments for summarize feature.
    """

    summarize_parser = sub_commands_parser.add_parser(
        "summarize",
        help="Check and display the changes list between two versions of a changelog file.",
    )

    summarize_parser.add_argument(
        "--old",
        help="Version from which we want the changes list. Default old version is the initial version of the changelog file.",
    )
    summarize_parser.add_argument(
        "--new",
        help="Version to which we want the changes list. Default new version is the latest version of the changelog file.",
    )
    # Add argument to include Unreleased section of changelog or not
    add_include_unreleased_argument(summarize_parser)
    # Add argument to be able to choose another path for changelog file
    add_changelog_path_argument(summarize_parser)

    return summarize_parser


def summarize(arguments: argparse.Namespace):
    """
    Return the changes list between two versions of a changelog file.
    """

    path = sanitize_changelog_path(Path(arguments.changelog_path))

    changelog_file = read_changelog_path(path)

    summaries, unassigned_items, old, new = get_summary(
        path, changelog_file, arguments.old, arguments.new, arguments.include_unreleased
    )

    print(f"# Changelog summary\n\nHere are all the changes between {old} and {new}:")
    # Display dict with changes sections sorted by alphabetic order
    for summary in sorted(summaries.keys()):
        print(f"\n{summary}\n")
        for change in summaries[summary]:
            print(change)
    if unassigned_items:
        print("\n/! WARNING: Following changes don't belong to any section:\n")
        for item in unassigned_items:
            print(item)


def main():
    import sys
    import argparse

    # Set up top level parser
    parser = argparse.ArgumentParser()

    # Set up sub_commands parser
    sub_commands = parser.add_subparsers(dest="command", title="Available commands")
    # Add all commands and their parameters
    add_get_parser(sub_commands)
    add_summarize_parser(sub_commands)

    arguments = parser.parse_args()

    try:
        if "get" == arguments.command:
            get(arguments)

        elif "summarize" == arguments.command:
            summarize(arguments)

        else:
            parser.print_help(sys.stderr)
            sys.exit(1)

    except Exception as e:
        print("An error occurred:", e, file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



import re
import argparse
from pathlib import Path
from typing import List


def add_changelog_path_argument(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    """
    Add a positional argument for the changelog path.
    """

    return parser.add_argument(
        "changelog_path",
        nargs="?",
        default=Path.cwd(),
        help="Path to the changelog file to process. Default path is the current directory. "
        "If the filename is not provided, it'll look for a CHANGELOG.md file.",
    )


def add_include_unreleased_argument(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    """
    Add an argument to be able to include Unreleased section when perform actions on a changelog file.
    """

    return parser.add_argument(
        "--include_unreleased",
        default=False,
        action="store_true",
        help="Include unreleased items to the output. By default, only released items are included.",
    )


def sanitize_changelog_path(changelog_path: Path) -> Path:
    """
    Check that the given path points to a valid file and if a file named CHANGELOG.md exists in the given directory.

    Return a valid changelog path.
    """

    if not changelog_path.is_file():
        changelog_path = changelog_path / "CHANGELOG.md"

    if not changelog_path.is_file():
        raise FileNotFoundError(f"The changelog file {changelog_path} doesn't exist.")

    return changelog_path


def read_changelog_path(changelog_path: Path) -> List[str]:
    """
    Open and read a changelog path.

    Return a splited changelog file.
    """

    changelog_file = changelog_path.read_text()

    return changelog_file.split("\n")


def get_version_line_pattern(changelog_format: str = "markdown") -> str:
    """
    Returns version line pattern depending on changelog file format.
    """

    if changelog_format == "markdown":
        return r"^## \[(.+)\]"


def get_version_pattern(version: str) -> str:
    """
    Use a provided version to create a pattern.
    """

    return rf"^## \[{re.escape(version)}\]"


def get_version(changelog_file: list, include_unreleased: bool, get_initial: bool = False) -> str:
    """
    Find and return initial version of a changelog file.
    """

    if get_initial:
        changelog_file = reversed(changelog_file)

    for line in changelog_file:
        current_version = re.search(get_version_line_pattern(), line)

        # If current_version isn't found or we found Unreleased section we continue, unless include_unreleased is set to True
        if current_version is None or (current_version.group(1) == "Unreleased" and not include_unreleased):
            continue
        return current_version.group(1)
    # If no version has been found, we throw an error.
    raise RuntimeError("Couldn't find a version line in the file: ")


def get_latest_version(changelog_file: list, include_unreleased: bool) -> str:
    """
    Return latest version of a changelog file.
    """

    return get_version(changelog_file, include_unreleased)


def get_initial_version(changelog_file: list, include_unreleased: bool) -> str:
    """
    Return initial version of a changelog file.
    """

    return get_version(changelog_file, include_unreleased, True)


def is_version_in_changelog(changelog_file: list, version: str) -> bool:
    """
    Check if given version exists in the given changelog file.
    """

    for line in changelog_file:
        if re.search(rf"^## \[{version}]", line) is None:
            continue
        return True

    return False


def is_version_line(line: str) -> bool:
    """
    Check if the line is a line that contains a version.
    """

    if line.startswith("## ["):
        return True

    return False


def is_section_line(line: str) -> bool:
    """
    Check if the line is a section heading.
    """

    if line.startswith("###"):
        return True

    return False



import re
from pathlib import Path
from .helpers import (
    is_version_in_changelog,
    get_latest_version,
    get_initial_version,
    get_version_pattern,
    is_version_line,
    is_section_line,
)
from typing import Dict, List


def generate_summary(changelog_file: list, top: str, bottom: str) -> Dict[str, List[str]]:
    """
    Summarise all the types of changes that have been made between two provided versions.

    Parameters
    - changelog_path: Path of the changelog we want to extract parts.
    - top: Version from which we want to start extract changes. This is the first version we'll need to find with for loop.
    - bottom: Version to which we want to stop extract changes. This is the last version we'll need to find with for loop.

    Return a dict with a short header that reminds top/bottom versions and all different kind of found sections, grouped by type of change.
    """

    current_section = None
    extract_on = False
    complete_on = False
    result = {}
    unassigned_items = []

    for line in changelog_file:
        # Everytime an empty line is found, it's ignored
        if line == "":
            continue
        # When we find the first version, we set a bool to True, which means we can begin to appends line in our dict
        if re.match(get_version_pattern(top), line):
            extract_on = True
        # When we met a new version, we set current_section to None, in case we met changes without sections
        if extract_on and is_version_line(line):
            current_section = None
        # When we find a heading section line, we can set current_version with current line
        if extract_on and is_section_line(line):
            current_section = line
        # Define current_section allow us to tell that all following lines can be added to the dict with current_section as a key
        if extract_on and not is_section_line(line) and not is_version_line(line):
            if current_section in result:
                result[current_section].append(line)
            # In case section header is missing, we still want to keep unassigned items
            elif current_section is None:
                unassigned_items.append(line)
            else:
                result[current_section] = [line]
        # When we find the other version, we set a bool to True, in order to continue adding following lines
        # until another version is found or the end of file is reached
        if re.match(get_version_pattern(bottom), line):
            complete_on = True
            continue
        if complete_on and is_version_line(line):
            break

    return result, unassigned_items


def get_summary(changelog_path: Path, changelog_file: list, old: str, new: str, include_unreleased: bool):
    """
    Set values for old and new versions and check if old/new versions exist in changelog file, otherwise raise an error.
    """

    # If old version is not provided, we set initial version as default value.
    if old is None:
        old = get_initial_version(changelog_file, include_unreleased)
    # If new version is not provided, we set latest version as default value.
    if new is None:
        new = get_latest_version(changelog_file, include_unreleased)
    # If versions are provided in invalid order, we raise an error
    if old > new:
        raise RuntimeError(
            f"Versions are provided in invalid order. Old version '{old}' shouldn't be higher than the new version '{new}'."
        )
    # If old version does not exist in the changelog file, we raise an error
    if not is_version_in_changelog(changelog_file, old):
        raise RuntimeError(f"Old version '{old}' does not exist in {changelog_path}.")
    # If new version does not exist in the changelog file, we raise an error
    if not is_version_in_changelog(changelog_file, new):
        raise RuntimeError(f"New version '{new}' does not exist in {changelog_path}.")

    # Now, that we are sure that we get correct versions, we can read the changelog file to get what we need
    summary, unassigned_items = generate_summary(changelog_file, new, old)

    return summary, unassigned_items, old, new



# get.py

import os
import sys
import unittest
from pathlib import Path


sys.path.append(os.path.join(os.path.dirname(__file__), "../src"))
from changelog_tools.changelog_helpers.helpers import (  # noqa 420
    get_latest_version,
    get_initial_version,
    is_version_in_changelog,
    read_changelog_path,
)


class GetVersionTest(unittest.TestCase):
    def setUp(self):
        self.semver_changelog_path = read_changelog_path(Path("data/CHANGELOG_SEM_VER.md"))
        self.no_semver_changelog_path = read_changelog_path(Path("data/CHANGELOG_NO_SEM_VER.md"))
        self.latest_semver_version = get_latest_version(self.semver_changelog_path, False)
        self.latest_no_semver_version = get_latest_version(self.no_semver_changelog_path, False)
        self.initial_semver_version = get_initial_version(self.semver_changelog_path, False)
        self.initial_no_semver_version = get_initial_version(self.no_semver_changelog_path, False)

    def test_changelog_file(self):
        """
        Test to get version with a file that isn't a changelog file.
        Test to get version without providing a file.
        """

        path = Path("README.md")

        with self.assertRaises(Exception):
            get_latest_version(path)

        with self.assertRaises(Exception):
            get_latest_version()

    def test_get_latest_version(self):
        """
        Test that we get the latest version number, even if it don't respect semantic versioning.
        """

        print("Test version number...")
        self.assertEqual(self.latest_semver_version, "1.0.3")
        self.assertEqual(self.latest_no_semver_version, "14.0.6-5A.5.0.0.202304182122")

    def test_get_initial_version(self):
        """
        Test that we get the initial version number, even if it don't respect semantic versioning.
        """

        self.assertEqual(self.initial_semver_version, "1.0.0")
        self.assertEqual(self.initial_no_semver_version, "1:2.30.2-1+deb11u2")

    def test_is_version_in_changelog(self):
        """
        Test that we check correctly if a given version exists or not.
        """

        self.assertFalse(is_version_in_changelog(self.semver_changelog_path, "0.0.0"))
        self.assertTrue(is_version_in_changelog(self.semver_changelog_path, "1.0.0"))


if __name__ == "__main__":
    unittest.main()



# summarize.py

import os
import sys
import unittest
from pathlib import Path


sys.path.append(os.path.join(os.path.dirname(__file__), "../src"))
from changelog_tools.changelog_helpers.summarize import get_summary  # noqa: E402
from changelog_tools.changelog_helpers.helpers import (  # noqa: E402
    read_changelog_path,
    is_section_line,
    is_version_line,
    get_version_pattern,
    get_version_line_pattern,
)


expected_dict = {
    "### Added": [
        "- v1.1 Brazilian Portuguese translation.",
        "- v1.1 German Translation",
        "- v1.1 Spanish translation.",
        "- v1.1 Italian translation.",
        "- v1.1 Polish translation.",
        "- v1.1 Ukrainian translation.",
        "- Version navigation.",
    ],
    "### Changed": [
        "- Use frontmatter title & description in each language version template",
        "- Replace broken OpenGraph image with an appropriately-sized Keep a Changelog image that will render properly (although in English for all languages)",
        "- Fix OpenGraph title & description for all languages so the title and description when links are shared are language-appropriate",
    ],
    "### Removed": ["- Trademark sign previously shown after the project description in version"],
    "### Fixed": [
        "- Improve French translation (#377).",
        "- Improve id-ID translation (#416).",
        "- Improve Persian translation (#457).",
    ],
}

expected_dict_with_unreleased = {
    "### Added": [
        '- Answer "Should you ever rewrite a change log?".',
        "- v1.1 Brazilian Portuguese translation.",
        "- v1.1 German Translation",
        "- v1.1 Spanish translation.",
        "- v1.1 Italian translation.",
        "- v1.1 Polish translation.",
        "- v1.1 Ukrainian translation.",
        "- Version navigation.",
    ],
    "### Changed": [
        "- Use frontmatter title & description in each language version template",
        "- Replace broken OpenGraph image with an appropriately-sized Keep a Changelog image that will render properly (although in English for all languages)",
        "- Fix OpenGraph title & description for all languages so the title and description when links are shared are language-appropriate",
    ],
    "### Removed": ["- Trademark sign previously shown after the project description in version"],
    "### Fixed": [
        "- Improve French translation (#377).",
        "- Improve id-ID translation (#416).",
        "- Improve Persian translation (#457).",
    ],
}


class SummarizeTest(unittest.TestCase):
    def setUp(self):
        self.semver_changelog_path = Path("data/CHANGELOG_SEM_VER.md")
        self.semver_changelog_file = read_changelog_path(self.semver_changelog_path)
        self.expected_unassigned_items = ['- New "Guiding Principles" sub-section to "How do I make a changelog?".']

    def test_summarize(self):
        """
        Test that we get all changes between two versions of a changelog file, grouped by changes type.
        """

        # Test that an error is raised if we provide a version that does not exist in the changelog file
        with self.assertRaises(Exception):
            get_summary(self.semver_changelog_path, "0.0.0", "1.0.1")
        # Test that an error is raised if we provide versions in invalid order
        with self.assertRaises(Exception):
            get_summary(
                self.semver_changelog_path,
                self.semver_changelog_file,
                "1.0.3",
                "1.0.1",
                False,
            )
        # Test that default version is set correctly if we provide only one version
        self.assertTrue(
            get_summary(
                self.semver_changelog_path,
                self.semver_changelog_file,
                None,
                "1.0.1",
                False,
            )
        )
        # Test that default versions are set correctly if no version is provided
        self.assertTrue(
            get_summary(
                self.semver_changelog_path,
                self.semver_changelog_file,
                None,
                None,
                False,
            )
        )
        # Test that default versions are set correctly if one version is provided and if we include Unreleased
        self.assertTrue(
            get_summary(
                self.semver_changelog_path,
                self.semver_changelog_file,
                None,
                "Unreleased",
                True,
            )
        )
        # Test that default versions are set correctly if no version is provided and if we include Unreleased
        self.assertTrue(get_summary(self.semver_changelog_path, self.semver_changelog_file, None, None, True))

        # Test that we return correct version line pattern for markdown changelog file
        self.assertTrue(get_version_line_pattern(), "^## [(.+)]")
        self.assertFalse(get_version_line_pattern("debian"), "^## [(.+)]")

        # Test that only string starting with ## [ are version line
        self.assertTrue(is_version_line("## [1.0.1]"))
        self.assertFalse(is_version_line("Version line without markdown heading."))
        # Test that only string starting with ### are section line
        self.assertTrue(is_section_line("### Added"))
        self.assertFalse(is_section_line("Section line without markdown heading."))

        # Test that get_version_pattern return expected pattern for the given version
        self.assertEqual(get_version_pattern("0.0.1"), "^## \\[0\\.0\\.1\\]")

        print("Test summary...")
        found_changes, unassigned_items, start, end = get_summary(
            self.semver_changelog_path,
            self.semver_changelog_file,
            "1.0.1",
            "1.0.3",
            False,
        )
        # Test that expected content without unreleased matches with found content
        self.assertEqual(expected_dict, found_changes)
        found_changes_with_unreleased, unassigned_items_with_unreleased, start, end = get_summary(
            self.semver_changelog_path,
            self.semver_changelog_file,
            "1.0.1",
            "Unreleased",
            True,
        )
        # Test that expected content with unreleased matches with found content
        self.assertEqual(expected_dict_with_unreleased, found_changes_with_unreleased)
        # Test that unassigned_items are stored correctly in a list
        self.assertEqual(self.expected_unassigned_items, unassigned_items_with_unreleased)


if __name__ == "__main__":
    unittest.main()
